import 'package:apollo_flutter/src/apollo_api.dart';
import 'package:apollo_flutter/apollo_theme.dart';
import 'package:apollo_flutter/src/apollo_theme_utils.dart';

class Apollo {
  static final _flutterApolloApi = FlutterApolloApi();
  static final Map<String, Apollo> _instances = {};

  static const _initializedApolloFailedDescription = "Initialized Apollo failed";

  static Future<Apollo> getInstance(String appName) async {
    if (_instances[appName] == null) {
      _instances[appName] = await _createNewInstance(appName);
    }

    return _instances[appName]!;
  }

  static Future<Apollo> _createNewInstance(String appName) async {
    final newApolloInstance = Apollo._(appName);
    try {
      await newApolloInstance._initialize();
    } on Exception catch (error) {
      throw Exception("$_initializedApolloFailedDescription: $error");
    } on Error catch (error) {
      throw Exception("$_initializedApolloFailedDescription: $error");
    }

    return newApolloInstance;
  }

  final String _appName;
  late ApolloTheme _theme;

  Apollo._(this._appName);

  Future<void> _initialize() async {
    final result = await _flutterApolloApi.getInstance(_appName);
    if (result.isSuccess != true) {
      throw Exception(result.error);
    }

    final rawColorTheme = result.colorTheme;
    if (rawColorTheme == null) {
      throw Exception("Missing theme config for Apollo");
    }

    _theme = ApolloTheme(colors: rawColorTheme.toApolloColorTheme());
  }

  ApolloTheme getTheme() {
    return _theme;
  }
}
